'	Calculate DSB & SSB stereo separation given a composite response

	#LANG "qb"		 ' Compile with FreeBASIC
	DEFINT I-N		 ' Integer variables begin with these letters
	CONST d2r = 1.745329E-02 ' Degree-to-radian conversion constant
	DECLARE FUNCTION sep (c!, p!, bx!, by!, x1!, y1!, x2!, y2!)
	DECLARE FUNCTION db (p!)
	DIM f(6), a(6), p(6), x(6), y(6)

	f(1) = 1		' Frequencies in kHz
	f(2) = 15
	f(3) = 23
	f(4) = 37
	f(5) = 39
	f(6) = 53

	WIDTH 73, 20

	PRINT "Stereo Separation Calculator"
	PRINT "Version 2.01 by Brian Beezley"
	PRINT
	PRINT "Calculates DSB and SSB stereo separation at 1 kHz and 15 kHz given the"
	PRINT "composite response. Calculates the audio response at 15 kHz relative"
	PRINT "to that at 1 kHz. Assumes an ideal transmit signal and stereo decoder."
	PRINT
	PRINT "Use a file that contains six lines with the composite amplitude in volts"
	PRINT "followed by the phase in degrees for these frequencies: 1, 15, 23, 37,"
	PRINT "39, and 53 kHz. If any amplitude is negative, all are assumed to be in"
	PRINT "dB. The program displays any text that follows the data."
	PRINT
	PRINT "The filename extension is .TXT if none entered."
	PRINT

	INPUT ; "File = ", a$
	a$ = UCASE$(a$)
	IF INSTR(a$, ".") = 0 THEN a$ = a$ + ".TXT"  ' Default file extension
	PRINT

	LOCATE , , 0			' Disable cursor

	IF OPEN(a$ FOR INPUT AS #1) THEN	' Open data file
	  PRINT a$; " not found"
	  SLEEP
	  END
	END IF

	CLS				' Clear screen
	PRINT a$			' Display filename
	PRINT

	FOR i = 1 TO 6			' Read composite amplitude and phase
	  INPUT #1, a(i), p(i)
	  IF a(i) < 0 THEN idb = -1	' Negative means amplitudes are in dB
	NEXT i

	IF idb THEN			' If dB, convert to linear amplitude
	  FOR i = 1 TO 6
	    a(i) = 10 ^ (.05 * a(i))
	  NEXT i
	END IF

	DO WHILE EOF(1) = 0		' Display any text following the data
      	  LINE INPUT #1, a$
	  IF a$ <> "" THEN
	    PRINT a$
	    k = -1
	  END IF
	LOOP
	IF k <> 0 THEN PRINT

' Display composite response in dB relative to 1-kHz response

	PRINT "    1    15    23    37    39    53  kHz"
	FOR i = 1 TO 6
	  PRINT USING "###.# "; 2 * db(a(i) / a(1)); ' db expects power ratio
	NEXT i					     ' 2 corrects voltage ratio
	PRINT " dB    Amplitude"

' Calculate a & b for least-squares fit to line y = cx + b
' Use only to display phase linearity

	FOR i = 1 TO 6
	  sx = sx + f(i)
	  sx2 = sx2 + f(i) * f(i)
	  sy = sy + p(i)
	  sxy = sxy + f(i) * p(i)
	NEXT i
	c = (6 * sxy - sx * sy) / (6 * sx2 - sx * sx)
	b = (sy - c * sx) / 6

' Display composite phase deviation

	FOR i = 1 TO 6
	  PRINT USING "+##.# "; p(i) - (c * f(i) + b);
	NEXT i
	PRINT " deg   Deviation from linear phase"
	PRINT

' Modulated L-R amplitudes

	FOR i = 3 TO 6
	  a(i) = .5 * a(i)	' DSB components have half amplitude
	NEXT i			' SSB amplitudes are doubled below

' Measured 38-kHz phase was always the average of that at 37 and 39 kHz.
' So use the average to avoid an extra data point.
' Using 38-kHz phase assumes that the tuner properly compensates for any
' excess phase shift between 19 and 38 kHz.

	p38 = .5 * (p(4) + p(5))	' 38-kHz phase

	p(3) = p38 - p(3)		' Detected L-R phase
	p(4) = p38 - p(4)
	p(5) = p(5) - p38
	p(6) = p(6) - p38

	FOR i = 1 TO 6	       		' Polar -> rectangular
	  x(i) = a(i) * COS(p(i) * d2r)
	  y(i) = a(i) * SIN(p(i) * d2r)
	NEXT i

' Calculate L+R coefficient that maximizes 1-kHz separation

	c = ((x(4) + x(5)) * x(1) + (y(4) + y(5)) * y(1)) / (a(1) * a(1))

' Calculate 1-kHz and 15-kHz separation

	dsb1 = sep(c, d1, x(1), y(1), x(4), y(4), x(5), y(5))
	ssb1 = sep(c, s1, x(1), y(1), x(4), y(4), x(4), y(4))	' Double LSB amp
	dsb15 = sep(c, d15, x(2), y(2), x(3), y(3), x(6), y(6))
	ssb15 = sep(c, s15, x(2), y(2), x(3), y(3), x(3), y(3))	' Double LSB amp

' Calculate 15-kHz response with respect to 1-kHz response

	d15 = db(d15 / d1)
	s15 = db(s15 / s1)

' Display results

	PRINT       "        Separation    Response"
	PRINT       "      1 kHz   15 kHz   15 kHz"
	PRINT USING "DSB   ###.#   ###.#    ###.# dB"; dsb1; dsb15; d15
	PRINT USING "SSB   ###.#   ###.#    +##.#"; ssb1; ssb15; s15
	PRINT
	PRINT USING "L+R coef = #.####"; c

	SLEEP

	FUNCTION sep (c, sum, bx, by, x1, y1, x2, y2)	' Stereo matrix
	x = c * bx + x1 + x2				' c represents stereo
	y = c * by + y1 + y2				' separation trimpot
	sum = x * x + y * y				' Left channel power
	x = c * bx - x1 - x2
	y = c * by - y1 - y2
	dif = x * x + y * y				' Right channel power
	sep = db(sum / dif)				' Separation in dB
	END FUNCTION

	FUNCTION db (p)					' Power ratio -> dB
	db = 10 * LOG(p) / LOG(10)			' LOG is natural log
	END FUNCTION